import java.io.*;
import java.util.*;

/**
 * @author Ilya Peresadin <pva701@gmail.com>
 * @created 14.10.16
 */
public class stringsqueries_ip2 implements Closeable {
    public static final int N = 200_000 + 10;
    public static final int SUM_INT = 1_000_000 + 10;
    FastScanner in;
    PrintWriter out;

    static class Ref<T> {
        T val;
        public Ref() {}
        public Ref(T v) {
            this.val = v;
        }
        public void set(T v) {
            val = v;
        }
        public T get() {
            return val;
        }
    }

    static class Node {
        static List<Integer> shuffle = new ArrayList<>();
        static int ptr;
        static Random random = new Random();
        static {
            for (int i = 0; i < N; ++i) {
                shuffle.add(i);
            }
            Collections.shuffle(shuffle, random);
        }

        int x, y, sum;
        String val;
        Node l, r;
        public Node(int x, String s) {
            this.x = x;
            this.val = s;
            y = shuffle.get(ptr++);
        }
    }

    int sum(Node v) {
        if (v != null) {
            return v.sum;
        } else {
            return 0;
        }
    }

    Node update(Node v) {
        if (v != null) {
            v.sum = sum(v.l) + sum(v.r) + v.val.length();
        }
        return v;
    }

    Node merge(Node L, Node R) {
        if (L == null) return update(R);
        if (R == null) return update(L);
        if (L.y > R.y) {
            L.r = merge(L.r, R);
            return update(L);
        } else {
            R.l = merge(L, R.l);
            return update(R);
        }
    }

    void splitBySum(Node t, int k, Ref<Node> L, Ref<Node> R) {
        if (t == null) {
            L.set(null);
            R.set(null);
        } else if (sum(t.l) + t.val.length() <= k) {
            splitBySum(t.r, k - sum(t.l) - t.val.length(), L, R);
            t.r = L.get();
            L.set(t);
        } else {
            splitBySum(t.l, k, L, R);
            t.l = R.get();
            R.set(t);
        }
        update(t);
    }

    void split(Node t, int k, Ref<Node> L, Ref<Node> R) {
        if (t == null) {
            L.set(null);
            R.set(null);
        } else if (t.x <= k) {
            split(t.r, k, L, R);
            t.r = L.get();
            L.set(t);
        } else {
            split(t.l, k, L, R);
            t.l = R.get();
            R.set(t);
        }
        update(t);
    }


    void getSubtreeStrings(Node root, String[] subtreeStrings, Ref<Integer> sz) {
        if (root == null) {
            return;
        }
        getSubtreeStrings(root.l, subtreeStrings, sz);
        subtreeStrings[sz.get()] = root.val;
        sz.set(sz.get() + 1);
        getSubtreeStrings(root.r, subtreeStrings, sz);
    }

    String firstString(Node root) {
        Node v = root;
        while (v.l != null) {
            v = v.l;
        }
        return v.val;
    }

    private String[] subtreeStrings = new String[N];
    private Ref<Integer> sz = new Ref<>(0);
    private Ref<Node> L = new Ref<>(), M = new Ref<>(), R = new Ref<>();

    String getSubstring(Ref<Node> t, int i, int j) {
        L.set(null);M.set(null);R.set(null);sz.set(0);
        splitBySum(t.get(), j, L, R);
        int maxLen = sum(L.get());
        splitBySum(L.get(), i - 1, L, M);
        StringBuilder answer = new StringBuilder();
        Node Ln = L.get(), Mn = M.get(), Rn = R.get();
        if (maxLen < i) {
            String oneString = firstString(Rn);
            i -= sum(Ln) + 1;
            j -= sum(Ln) + 1;
            answer.append(oneString.substring(i, j + 1));
        } else {
            int prefLen = sum(Mn) - (maxLen - i + 1);
            getSubtreeStrings(Mn, subtreeStrings, sz);
            String first = subtreeStrings[0];
            answer.append(first.substring(prefLen));
            int sizeSubtree = sz.get();
            for (int idx = 1; idx < sizeSubtree; ++idx) {
                answer.append(subtreeStrings[idx]);
            }
            if (maxLen != j) {
                String lastString = firstString(Rn);
                answer.append(lastString.substring(0, j - maxLen));
            }
        }
        t.set(merge(merge(L.get(), M.get()), R.get()));
        return answer.toString();
    }

    Node insert(Node root, int x, String val) {
        Node Mn = new Node(x, val);
        split(root, x, L, R);
        return merge(merge(L.get(), Mn), R.get());
    }

    Node remove(Node t, int x) {
        split(t, x, L, R);
        split(L.get(), x - 1, L, M);
        return merge(L.get(), R.get());
    }

    class Event implements Comparable<Event> {
        int type, x, i;
        Event() {}
        Event(int type, int x, int i) {
            this.type = type;
            this.x = x;
            this.i = i;
        }

        @Override
        public int compareTo(Event ev) {
            if (x == ev.x) {
                return type - ev.type;
            }
            return x - ev.x;
        }
    };



    public void run() {
        int n = in.nextInt();
        int m = in.nextInt();

        Event[] ev = new Event[2 * m];
        int events = 0;
        String[] strings = new String[m];
        String[] answers = new String[m];
        int[] x = new int[m];
        int[] y = new int[m];

        for (int i = 0; i < m; ++i) {
            int tp = in.nextInt();
            if (tp == 1) {
                int l = in.nextInt();
                int r = in.nextInt();
                String s = in.next();
                strings[i] = s;
                ev[events++] = new Event(0, l, i);
                ev[events++] = new Event(2, r, i);
            } else {
                int p = in.nextInt();
                int l = in.nextInt();
                int r = in.nextInt();
                x[i] = l;
                y[i] = r;
                ev[events++] = new Event(1, p, i);
            }
        }

        Node root = null;
        Ref<Node> refRoot = new Ref<>();
        Arrays.sort(ev, 0, events);
        for (int i = 0; i < events; ++i) {
            if (ev[i].type == 0) {
                root = insert(root, ev[i].i, strings[ev[i].i]);
            } else if (ev[i].type == 2) {
                root = remove(root, ev[i].i);
            } else {
                int id = ev[i].i;
                refRoot.set(root);
                answers[id] = getSubstring(refRoot, x[id], y[id]);
                root = refRoot.get();
            }
        }

        for (int i = 0; i < m; ++i)
            if (answers[i] != null) {
                out.println(answers[i]);
            }
    }

    public stringsqueries_ip2() throws IOException {
        in = new FastScanner(new File("stringsqueries.in"));
        out = new PrintWriter("stringsqueries.out");
    }

    @Override
    public void close() throws IOException {
        out.close();
    }

    public static void main(String[] args) throws IOException {
        stringsqueries_ip2 instance = new stringsqueries_ip2();
        instance.run();
        instance.close();
    }

    class FastScanner {
        BufferedReader br;
        StringTokenizer st;

        FastScanner(File f) {
            try {
                br = new BufferedReader(new FileReader(f));
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
        }

        String next() {
            while (st == null || !st.hasMoreTokens()) {
                try {
                    st = new StringTokenizer(br.readLine());
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            return st.nextToken();
        }

        int nextInt() {
            return Integer.parseInt(next());
        }
    }
}
